using System;
using System.Collections.Generic;
using System.Linq;
using System.Threading.Tasks;
using MongoDB.Driver;
using Posts.WebApi.Models.Entities;
using Posts.WebApi.Models.Enums;
using Posts.WebApi.Models.Models;

namespace Posts.WebApi.DataAccess
{
    public class PostRepository : BaseMongoDbRepository<Post>, IPostRepository
    {
        const int INITIAL_POSTS_COUNT = 13;

        public PostRepository(IMongoDatabase mongoDatabase) : base(mongoDatabase)
        {
        }

        public async Task<IEnumerable<Post>> GetPostsBySubjectAsync(int subjectId,
            int groupId = 0, int? semester = 0, int? course = 0, int skip = 0, int take = 0)
        {
            var posts = await this._mongoDatabase.GetCollection<Post>(typeof(Post).Name).Find(p => p.SubjectId == subjectId)
                .SortByDescending(s => s.ModifiedAtUtc)
                .Skip(skip)
                .Limit(take)
                .ToListAsync();

            IEnumerable<Post> postEnumerable = posts;

            Func<Post, bool> predicate = p => p.SubjectId == subjectId;

            var postsQuery = this._mongoDatabase.GetCollection<Post>(typeof(Post).Name).Find(p => p.SubjectId == subjectId && p.Course == course && p.GroupId == groupId && p.Semester == semester);

            postsQuery = postsQuery.SortByDescending(s => s.ModifiedAtUtc);

            if (skip != 0)
            {
                postsQuery = postsQuery.Skip(skip);
            }

            if (take != 0)
            {
                postsQuery = postsQuery.Limit(take);
            }

            return await postsQuery.ToListAsync();
        }

        public async Task<IEnumerable<PostBySemesterGroup>> GetInitialGroupedPostsBySubjectAsync(int subjectId,
            string title = "", int groupId = 0, int? semester = 0, int? course = 0, EPostValueType? valueType = null,
            DateTimeOffset? givenDateFrom = null, DateTimeOffset? givenDateTo = null)
        {
            var posts = await this._mongoDatabase.GetCollection<Post>(typeof(Post).Name).Find(p => p.SubjectId == subjectId && p.DeletedAtUtc == null)
                                               .SortByDescending(p => p.GroupId).ToListAsync();

            var postsGrouped = posts.GroupBy(p => new { p.GroupId, p.Semester, p.Course }, p => p)
                                                      .Select(g => new PostBySemesterGroup
                                                      {
                                                          GroupId = g.Key.GroupId,
                                                          Semester = g.Key.Semester,
                                                          GroupName = $" ",
                                                          Course = g.Key.Course,
                                                          Posts = g
                                                                              .OrderByDescending(p => p.ModifiedAtUtc)
                                                                              .Take(INITIAL_POSTS_COUNT).ToList()
                                                      });

            return postsGrouped;
        }


        public async Task<IEnumerable<Post>> GetFullUsersPostAsync(int userId, int skip, int take)
        {
            var posts = this._mongoDatabase.GetCollection<Post>(typeof(Post).Name).Find(p => p.UserId == userId);

            if (skip != 0)
            {
                posts = posts.Skip(skip);
            }

            if (take != 0)
            {
                posts = posts.Limit(take);
            }

            return await posts.ToListAsync();
        }
    }
}