import { Component, OnInit, AfterViewInit, OnDestroy } from '@angular/core';
import { UntypedFormGroup, Validators, UntypedFormControl } from '@angular/forms';

import { ValidationService, PopupService } from '@services/index';
import { Store } from '@ngrx/store';
import * as fromStore from '../../model/store/index';
import { Observable } from 'rxjs';
import { map, withLatestFrom } from 'rxjs/operators';
import { ClearErrorMessage, ClearRegistrationStatus } from '../../model/store/index';
import { Router } from '@angular/router';
import { UntilDestroy, untilDestroyed } from '@ngneat/until-destroy';

@UntilDestroy()
@Component({
  selector: 'app-register',
  templateUrl: './register.component.html',
  styleUrls: ['./register.component.scss']
})
export class RegisterComponent implements OnInit, AfterViewInit, OnDestroy {

  private successMessage = "Success! Validate your email now!";
  public loading$: Observable<boolean>;

  public registerForm = new UntypedFormGroup({
    emailInput: new UntypedFormControl('', [Validators.email, Validators.required]),
    passwordInput: new UntypedFormControl('', [Validators.minLength(6), Validators.maxLength(32), Validators.required]),
    passwordConfirmInput: new UntypedFormControl('', [Validators.required]),
    usernameInput: new UntypedFormControl('', [Validators.required,
    Validators.minLength(5), Validators.maxLength(32), Validators.pattern("^[a-zA-Z0-9_.]*$")]),
  });

  constructor(
    private validator: ValidationService,
    private popupService: PopupService,
    private router: Router,
    private store: Store<fromStore.AuthenticationState>
  ) {
    this.loading$ = store.select(fromStore.getLoading);

    store.select(fromStore.getErrorMessage).pipe(untilDestroyed(this))
      .subscribe((value: string | undefined) => {
        if (value) {
          this.registerFailure(value);
        }
      });

    store.select(fromStore.getLoaded)
      .pipe(untilDestroyed(this),
        withLatestFrom(store.select(fromStore.getRegistraionResult)),
        map(([isLoaded, isRegistrationSuccessful]) => {
          if (isLoaded === true && isRegistrationSuccessful === true) {
            return isRegistrationSuccessful;
          }
          return undefined;
        })).subscribe(value => this.registerSuccess(value));
  }


  public ngOnInit() { }

  public ngAfterViewInit() {
    this.registerForm.controls['passwordConfirmInput'].setValidators(
      this.validator.passwordMatchValidator(
        this.registerForm.controls['passwordInput'],
        this.registerForm.controls['passwordConfirmInput']));
  }

  public registerButton() {
    const emailInput = this.registerForm.controls['emailInput'];
    const passwordInput = this.registerForm.controls['passwordInput'];
    const passwordConfirmInput = this.registerForm.controls['passwordConfirmInput'];
    const usernameInput = this.registerForm.controls['usernameInput'];

    if (emailInput.invalid) {
      this.popupService.showError(this.validator.getEmailErrorMessage(emailInput));
      return;
    }
    if (passwordInput.invalid) {
      this.popupService.showError(this.validator.getPasswordErrorMessage(passwordInput));
      return;
    }
    if (passwordConfirmInput.invalid) {
      this.popupService.showError(this.validator.getConfirmPasswordErrorMessage(passwordConfirmInput));
      return;
    }
    if (usernameInput.invalid) {
      this.popupService.showError(this.validator.getUsernameErrorMessage(usernameInput));
      return;
    }

    this.store.dispatch(new fromStore.RegistrationStart(usernameInput.value,
      emailInput.value,
      passwordInput.value));
  }

  public ngOnDestroy(): void { }

  private registerSuccess(isRegistrationSuccessful: boolean | undefined): void {
    if (isRegistrationSuccessful === true) {
      this.router.navigate(['authorization']);
      this.popupService.showMessage(this.successMessage);
      this.store.dispatch(new ClearRegistrationStatus());
    }
  }

  private registerFailure(error: string | undefined): void {
    if (error)
      this.popupService.showError(error);
    this.store.dispatch(new ClearErrorMessage());
  }
}
