using System.Collections.Generic;
using System.Security.Claims;
using System.Threading.Tasks;
using System.Web;
using AutoMapper;
using Microsoft.Extensions.Logging;
using Microsoft.Extensions.Options;
using Uni.WebApi.ApplicationCore.Constants;
using Uni.WebApi.ApplicationCore.Helpers;
using Uni.WebApi.ApplicationCore.Helpers.Contract;
using Uni.WebApi.ApplicationCore.Services.Contract;
using Uni.WebApi.ApplicationCore.Services.Shared.Contract;
using Uni.WebApi.Common.Options;
using Uni.WebApi.Common.Token;
using Uni.WebApi.DataAccess.RepositoryService;
using Uni.WebApi.Kernel.Entities;
using Uni.WebApi.Models.Enums;
using Uni.WebApi.Models.ModelDto;
using Uni.WebApi.Models.Models;
using Uni.WebApi.Models.Requests.Authorization;

namespace Uni.WebApi.ApplicationCore.Services
{
    public class AuthorizationService : IAuthorizationService
    {
        private readonly IUnitOfWork _unitOfWork;
        private readonly IRefreshTokenService _refreshTokenService;
        private readonly IEmailService _emailService;
        private readonly IMapper _mapper;
        private readonly UrlsOptions _urlOptions;
        private readonly ILogger<AuthorizationService> _logger;
        private readonly IDateHelper _dateHelper;
        private readonly IEncryptHelper _encryptHelper;

        public AuthorizationService(
            IUnitOfWork repositoryWrapper,
            IMapper mapper,
            IEmailService emailService,
            IRefreshTokenService refreshTokenService,
            IOptions<UrlsOptions> urlOptions,
            ILogger<AuthorizationService> logger,
            IDateHelper dateHelper,
            IEncryptHelper encryptHelper)
        {
            this._encryptHelper = encryptHelper;
            this._dateHelper = dateHelper;
            this._emailService = emailService;
            this._mapper = mapper;
            this._unitOfWork = repositoryWrapper;
            this._refreshTokenService = refreshTokenService;
            this._urlOptions = urlOptions.Value;
            this._logger = logger;
        }

        public async Task<ServiceResult<AuthDto>> LoginAsync(LoginUserRequest request)
        {
            request.Email = request.Email.ToLowerInvariant();

            this._logger.LogInformation("-- LoginAsync -- email: " + request.Email);

            var userInfo = await this._unitOfWork.UserRepository.GetUserAsync(request.Email, true);

            if (userInfo == null || !Authenticate.Verify(request.Password, userInfo.PasswordHash))
            {
                return ServiceResult<AuthDto>.Fail(EOperationResult.ValidationError,
                    "Email or password is incorrect");
            }

            if (!userInfo.IsValidated)
            {
                return ServiceResult<AuthDto>.Fail(EOperationResult.ValidationError,
                    "You need to verify your email first");
            }

            userInfo.LastVisitedAtUtc = this._dateHelper.GetDateTimeUtcNow();
            await this._unitOfWork.CommitAsync();

            return ServiceResult<AuthDto>.Ok(await GenerateToken(userInfo));
        }

        public async Task<ServiceResult<object>> RegisterStudentAsync(RegisterUserRequest request)
        {
            request.Email = request.Email.ToLowerInvariant();
            request.Username = request.Username.ToLowerInvariant();

            if (await this._unitOfWork.UserRepository.AnyAsync(u =>
                u.Email == request.Email))
            {
                return ServiceResult<object>.Fail(EOperationResult.AlreadyExist, "User with this email already exist");
            }

            if (await this._unitOfWork.UserRepository.IsUserExistByUsernameAsync(request.Username))
            {
                return ServiceResult<object>.Fail(EOperationResult.AlreadyExist,
                    "User with this username already exist");
            }

            string encryptedString = this._encryptHelper.Encrypt(request.Email);

            var confirmationUrl =
                $"{this._urlOptions.ServerUrl}/{this._urlOptions.ApiVersion}/authorization/email-confirmation/{HttpUtility.UrlEncode(encryptedString)}";

            var sendEmailResult =
                await this._emailService.SendUserValidationEmailAsync(request.Email, request.Username, confirmationUrl);

            if (!sendEmailResult.IsSuccess)
            {
                return ServiceResult<object>.Fail(EOperationResult.SendEmailError,
                    $"Error while sending email with status code: {sendEmailResult.Result}.");
            }

            var user = new User()
                       {
                           Email = request.Email,
                           CreatedAtUtc = this._dateHelper.GetDateTimeUtcNow(),
                           PasswordHash = Authenticate.Hash(request.Password),
                           Username = request.Username,
                           RoleId = (int)ERoleType.Student,
                           Avatar = this._urlOptions.ServerUrl + DefaultImagesConstants.DefaultUserImage,
                           IsValidated = true
                       };

            this._unitOfWork.UserRepository.Add(user);
            await this._unitOfWork.CommitAsync();

            return ServiceResult<object>.Ok();
        }

        public async Task<ServiceResult<AuthDto>> ConfirmEmailAsync(string emailToken)
        {
            this._encryptHelper.TryDecrypt(emailToken, out string email);

            email = email.ToLowerInvariant();

            var user = await this._unitOfWork.UserRepository.GetSingleAsync(u => u.Email == email);

            if (user == null)
            {
                return ServiceResult<AuthDto>.Fail(EOperationResult.EntityNotFound,
                    "Invalid email verification token.");
            }

            user.IsValidated = true;
            user.ModifiedAtUtc = this._dateHelper.GetDateTimeUtcNow();

            await this._unitOfWork.CommitAsync();

            var authDto = await GenerateToken(user);

            return ServiceResult<AuthDto>.Ok(authDto);
        }

        public async Task<ServiceResult<TokenModel>> UpdateTokenAsync(RefreshTokenRequest refreshToken)
        {
            var oldRefreshToken = (await this._unitOfWork.RefreshTokenRepository.GetSingleAsync(
                r => r.Token == refreshToken.RefreshToken,
                r => r.User));

            if (oldRefreshToken?.User == null)
            {
                return ServiceResult<TokenModel>.Fail(EOperationResult.ValidationError, "Refresh token is invalid");
            }

            var isRefreshTokenValid =
                await this._refreshTokenService.IsValidRefreshTokenAsync(refreshToken.RefreshToken);
            if (!isRefreshTokenValid)
            {
                return ServiceResult<TokenModel>.Fail(EOperationResult.ValidationError, "Refresh token is invalid");
            }

            List<KeyValuePair<object, object>> keyValueList = new List<KeyValuePair<object, object>>
                                                              {
                                                                  new KeyValuePair<object, object>(
                                                                      SetOfKeysForClaims.UserId,
                                                                      oldRefreshToken.User.Id)
                                                              };

            return ServiceResult<TokenModel>.Ok(
                await this._refreshTokenService.GetTokenModelAsync(keyValueList, oldRefreshToken.UserId,
                    refreshToken.RefreshToken));
        }

        private async Task<AuthDto> GenerateToken(User user)
        {
            List<KeyValuePair<object, object>> keyValueList = new List<KeyValuePair<object, object>>
                                                              {
                                                                  new KeyValuePair<object, object>(
                                                                      SetOfKeysForClaims.UserId, user.Id),
                                                                  new KeyValuePair<object, object>(ClaimTypes.Role,
                                                                      (ERoleType)user.RoleId)
                                                              };

            TokenModel tokenModel = await this._refreshTokenService.GetTokenModelAsync(keyValueList, user.Id);

            return new AuthDto()
                   {
                       AccessToken = tokenModel.AccessToken,
                       RefreshToken = tokenModel.RefreshToken,
                       ExpireAt = tokenModel.ExpireAt,
                       User = this._mapper.Map<User, UserDto>(user)
                   };
        }
    }
}