using System;
using System.Collections.Generic;
using System.Security.Cryptography;
using System.Threading.Tasks;
using Microsoft.Extensions.Options;
using Uni.WebApi.ApplicationCore.Helpers.Contract;
using Uni.WebApi.ApplicationCore.Services.Shared.Contract;
using Uni.WebApi.Common.Options;
using Uni.WebApi.Common.Token;
using Uni.WebApi.DataAccess.RepositoryService;
using Uni.WebApi.Kernel.Entities;

namespace Uni.WebApi.ApplicationCore.Services.Shared
{
    public class RefreshTokenService : IRefreshTokenService
    {
        private readonly IUnitOfWork _unitOfWork;
        private readonly IOptions<TokenOptions> _tokenOptions;
        private readonly ITokenService _tokenService;
        private readonly IDateHelper _dateHelper;

        public RefreshTokenService(
            IUnitOfWork unitOfWork,
            IOptions<TokenOptions> jwtTokenOptions,
            ITokenService tokenProvider,
            IDateHelper dateHelper)
        {
            this._dateHelper = dateHelper;
            this._unitOfWork = unitOfWork;
            this._tokenOptions = jwtTokenOptions;
            this._tokenService = tokenProvider;
        }

        public async Task<TokenModel> GetTokenModelAsync(IEnumerable<KeyValuePair<object, object>> keyValues, int userId,
            string oldRefreshToken = "")
        {
            var accessToken = this._tokenService.GetTokenModel(keyValues);
            string refreshToken = GetRefreshToken();

            await PutRefreshTokenAsync(userId, refreshToken, oldRefreshToken);

            return new TokenModel
            {
                RefreshToken = refreshToken,
                AccessToken = accessToken.AccessToken,
                ExpireAt = accessToken.ExpireAt
            };
        }

        public async Task<bool> IsValidRefreshTokenAsync(string refreshToken)
        {
            return await this._unitOfWork.RefreshTokenRepository.AnyAsync(x =>
                x.Token == refreshToken && x.ExpiredAt >= this._dateHelper.GetDateTimeUtcNow());
        }

        private string GetRefreshToken()
        {
            var bytesArray = new byte[32];
            using (var rng = RandomNumberGenerator.Create())
            {
                rng.GetBytes(bytesArray);
                return Convert.ToBase64String(bytesArray);
            }
        }

        private async Task PutRefreshTokenAsync(int userId, string refreshToken, string oldRefreshToken = "")
        {
            var dbRefreshToken = await this._unitOfWork.RefreshTokenRepository.GetSingleAsync(x => x.UserId == userId);
            if (dbRefreshToken == null)
            {
                var refreshTokenEntity = new RefreshToken
                {
                    UserId = userId,
                    Token = refreshToken,
                    ExpiredAt = this._dateHelper.GetDateTimeUtcNow().AddMinutes(this._tokenOptions.Value.RefreshTokenLifeTime),
                    CreatedAtUtc = this._dateHelper.GetDateTimeUtcNow(),
                    ModifiedAtUtc = this._dateHelper.GetDateTimeUtcNow()
                };
                await this._unitOfWork.RefreshTokenRepository.AddAsync(refreshTokenEntity);
            }
            else
            {
                dbRefreshToken.Token = refreshToken;
                dbRefreshToken.ExpiredAt = this._dateHelper.GetDateTimeUtcNow().AddMinutes(this._tokenOptions.Value.RefreshTokenLifeTime);
                dbRefreshToken.ModifiedAtUtc = this._dateHelper.GetDateTimeUtcNow();
            }

            await this._unitOfWork.CommitAsync();
        }
    }
}