using System.Collections.Generic;
using System.Linq;
using System.Threading.Tasks;
using AutoMapper;
using Microsoft.Extensions.Options;
using Uni.WebApi.ApplicationCore.Constants;
using Uni.WebApi.ApplicationCore.Helpers.Contract;
using Uni.WebApi.ApplicationCore.Services.Contract;
using Uni.WebApi.Common.Options;
using Uni.WebApi.DataAccess.RepositoryService;
using Uni.WebApi.Kernel.Entities;
using Uni.WebApi.Models.ModelDto;
using Uni.WebApi.Models.Models;
using Uni.WebApi.Models.Requests;

namespace Uni.WebApi.ApplicationCore.Services
{
    public class SubjectService : ISubjectService
    {
        private readonly IUnitOfWork _unitOfWork;
        private readonly IMapper _mapper;
        private readonly UrlsOptions _urlOptions;
        private readonly IDateHelper _dateHelper;

        public SubjectService(
            IUnitOfWork unitOfWork,
            IOptions<UrlsOptions> urlOptions,
            IMapper mapper,
            IDateHelper dateHelper)
        {
            this._mapper = mapper;
            this._unitOfWork = unitOfWork;
            this._urlOptions = urlOptions.Value;
            this._dateHelper = dateHelper;
        }

        public async Task<ServiceResult<SubjectDto>> CreateSubjectAsync(AddSubjectRequest request)
           {
               if (!await this._unitOfWork.FacultyRepository.IsExistById(request.FacultyId))
               {
                   return ServiceResult<SubjectDto>.Fail(EOperationResult.EntityNotFound,
                       "Faculty with this Id doesn't exist");
               }

               if (!await this._unitOfWork.TeacherRepository.IsExistById(request.TeacherId))
               {
                   return ServiceResult<SubjectDto>.Fail(EOperationResult.EntityNotFound,
                       "Teacher with this Id doesn't exist");
               }

               var newSubject = new Subject()
               {
                   Title = request.Title,
                   TeacherId = request.TeacherId,
                   Description = request.Description,
                   FacultyId = request.FacultyId,
                   Avatar = request.Avatar,
                   CreatedAtUtc = this._dateHelper.GetDateTimeUtcNow(),
                   ModifiedAtUtc = this._dateHelper.GetDateTimeUtcNow()
               };

               if (string.IsNullOrEmpty(newSubject.Avatar))
               {
                   newSubject.Avatar = this._urlOptions.ServerUrl + DefaultImagesConstants.DefaultImage;
               }

               this._unitOfWork.SubjectRepository.Add(newSubject);

               await this._unitOfWork.CommitAsync();

               return ServiceResult<SubjectDto>.Ok(this._mapper.Map<Subject, SubjectDto>(newSubject));
           }

        public async Task<ServiceResult<IEnumerable<SubjectDto>>> GetListOfSubjectsAsync(int facultyId, int skip, int take)
        {
            IEnumerable<SubjectDto> result =
             (await this._unitOfWork.SubjectRepository.GetSubjectsByFacultyWithTeachersAsync(facultyId, skip, take))
                                                .Select(s => new SubjectDto()
                                                {
                                                    Id = s.Id,
                                                    Title = s.Title,
                                                    Description = s.Description,
                                                    Avatar = s.Avatar,
                                                    TeacherLastName = s.Teacher.LastName
                                                });

            return ServiceResult<IEnumerable<SubjectDto>>.Ok(result);
        }
    }
}