using System.Collections.Generic;
using System.Threading.Tasks;
using AutoMapper;
using Microsoft.Extensions.Options;
using Uni.WebApi.ApplicationCore.Constants;
using Uni.WebApi.ApplicationCore.Services.Contract;
using Uni.WebApi.Common.Options;
using Uni.WebApi.DataAccess.RepositoryService;
using Uni.WebApi.Kernel.Entities;
using Uni.WebApi.Models.ModelDto;
using Uni.WebApi.Models.Models;
using Uni.WebApi.Models.Requests;

namespace Uni.WebApi.ApplicationCore.Services
{
    public class UniversityService : IUniversityService
    {
        private readonly IUnitOfWork _unitOfWork;
        private readonly IMapper _mapper;
        private readonly UrlsOptions _urlOptions;

        public UniversityService(
            IUnitOfWork unitOfWork,
            IOptions<UrlsOptions> urlOptions,
            IMapper mapper)
        {
            this._mapper = mapper;
            this._unitOfWork = unitOfWork;
            this._urlOptions = urlOptions.Value;
        }

        public async Task<ServiceResult<UniversityDto>> CreateUniversityAsync(AddUniversityRequest request)
           {
               if (!await this._unitOfWork.CityRepository.IsExistById(request.CityId))
               {
                   return ServiceResult<UniversityDto>.Fail(EOperationResult.EntityNotFound, "City with this Id doesn't exist");
               }

               var newUniversity = new University()
               {
                   FullTitle = request.FullTitle,
                   ShortTitle = request.ShortTitle,
                   Description = request.Description,
                   CityId = request.CityId,
                   Avatar = DefaultImagesConstants.DefaultImage
               };

               if (string.IsNullOrEmpty(newUniversity.Avatar))
               {
                   newUniversity.Avatar = DefaultImagesConstants.DefaultImage;
               }

               this._unitOfWork.UniversityRepository.Add(newUniversity);

               await this._unitOfWork.CommitAsync();

               return ServiceResult<UniversityDto>.Ok(this._mapper.Map<University, UniversityDto>(newUniversity));
           }

        public async Task<ServiceResult<IEnumerable<UniversityDto>>> GetListOfUniversitiesAsync(int cityId, int skip, int take)
        {
            IEnumerable<UniversityDto> result;

            if (cityId != 0)
            {
                result = this._mapper.Map<IEnumerable<University>, List<UniversityDto>>(
                            await this._unitOfWork.UniversityRepository.GetUniversitiesByCityAsync(cityId, skip, take));
            }
            else
            {
                result = this._mapper.Map<IEnumerable<University>, List<UniversityDto>>(
                            await this._unitOfWork.UniversityRepository.GetAllUniversitiesAsync(skip, take));
            }

            return ServiceResult<IEnumerable<UniversityDto>>.Ok(result);
        }
    }
}