﻿using System;
using System.IO;
using Microsoft.AspNetCore.Builder;
using Microsoft.AspNetCore.Hosting;
using Microsoft.AspNetCore.Mvc;
using Microsoft.Extensions.Configuration;
using Microsoft.Extensions.DependencyInjection;
using Microsoft.Extensions.FileProviders;
using Microsoft.Extensions.Hosting;
using Microsoft.Extensions.Logging;
using Microsoft.Extensions.Options;
using Serilog;
using Serilog.Exceptions;
using Serilog.Sinks.Elasticsearch;
using Uni.WebApi.ApplicationCore.Helpers;
using Uni.WebApi.ApplicationCore.Helpers.Contract;
using Uni.WebApi.Common.Options;
using Uni.WebApi.Common.Token;
using Uni.WebApi.DataAccess;
using Uni.WebApi.DataAccess.RepositoryService;
using Uni.WebApi.Web.Extensions.StartupExtensions;
using Uni.WebApi.Web.Helpers.Mappers;

namespace Uni.WebApi
{
    public class Startup
    {
        private readonly IConfiguration _configuration;

        public Startup(IConfiguration configuration)
        {
            this._configuration = configuration;
        }

        public IConfiguration Configuration { get; }

        public void ConfigureServices(IServiceCollection services)
        {
            services.AddControllers().AddNewtonsoftJson();

            // TODO: move logging configuraitions in 
            string elastisearchUri;
            if (this._configuration["usedocker"] == "true")
            {
                services.AddDockerDbContext(this._configuration);
                elastisearchUri = this._configuration["ElasticConfiguration:UriDocker"];
            }
            else
            {
                services.AddDebugDbContext(this._configuration);
                elastisearchUri = this._configuration["ElasticConfiguration:Uri"];
            }

            var serilogConfiguration = new LoggerConfiguration()
                                      .Enrich.FromLogContext()
                                      .Enrich.WithExceptionDetails()
                                      .WriteTo.Elasticsearch(new ElasticsearchSinkOptions(new Uri(elastisearchUri))
                                                             {
                                                                 AutoRegisterTemplate = true,
                                                             });

            Log.Logger = serilogConfiguration.CreateLogger();
            AppDomain.CurrentDomain.DomainUnload += (o, e) => Log.CloseAndFlush();

            services.AddLogging(loggingBuilder =>
            {
                loggingBuilder.AddDebug();
                loggingBuilder.AddSerilog();
            });

            services.AddRouting(options => options.LowercaseUrls = true);

            services.Configure<SendGridOptions>(this._configuration.GetSection("SendGrid"));
            services.Configure<FilesOptions>(this._configuration.GetSection("Files"));
            services.Configure<UrlsOptions>(this._configuration.GetSection("Urls"));

            services.AddRepositories();
            services.AddTransient<IUnitOfWork, UnitOfWork>();

            services.AddTransient(typeof(SeedDatabase));

            services.AddServiceLayer();

            services.AddTransient<IEmailTemplatePicker, MemoryEmailTemplatePicker>();
            services.AddTransient<IFolderHelper, FolderHelper>();
            services.AddTransient<IDateHelper, DateHelper>();
            services.AddTransient<IEncryptHelper, EncryptHelper>();

            services.AddTransient<IServiceResultMapper, ServiceResultMapper>();
            services.AddTransient<ITokenService, TokenService>();

            services.AddSwagger();

            services.AddAutoMapper(typeof(Startup).Assembly);

            services.AddJwtAuth(this._configuration);

            services.AddApiVersioning(
                o =>
                {
                    o.AssumeDefaultVersionWhenUnspecified = true;
                    o.DefaultApiVersion = new ApiVersion(1, 0);
                });
        }

        public void Configure(IApplicationBuilder app,
            SeedDatabase seedDatabase,
            IWebHostEnvironment env,
            IFolderHelper folderHelper,
            IOptions<FilesOptions> filesOptions,
            ILoggerFactory loggerFactory)
        {
            loggerFactory.AddSerilog();
            
            //TODO add only friendly origins
            app.UseCors(options =>
            {
                options.AllowAnyMethod();
                options.AllowAnyHeader();
                options.AllowAnyOrigin();
            });
            
            app.UseAuthentication();
            
            // app.UseExceptionMiddleware();

            if (env.IsDevelopment())
            {
                app.UseDeveloperExceptionPage();
            }
            else
            {
                // The default HSTS value is 30 days. You may want to change this for production scenarios, see https://aka.ms/aspnetcore-hsts.
            }

            folderHelper.CreateFilesFoldersIfNotExist();

            seedDatabase.Seed();

            //app.UseHttpsRedirection();

            app.UseStaticFiles();
            app.UseStaticFiles(new StaticFileOptions
                               {
                                   FileProvider = new PhysicalFileProvider(
                                       Path.Combine(Directory.GetCurrentDirectory(), "Files")),
                                   RequestPath = "/Files"
                               });
            
            app.UseSwagger();
            
            app.UseSwaggerUI(c =>
            {
                c.SwaggerEndpoint("/swagger/v1/swagger.json", "Uni API V1");
            });

            app.UseRouting();
            app.UseAuthorization();
            app.UseEndpoints(endpoints =>
            {
                endpoints.MapControllers();
            });
        }
    }
}